using System.Collections.Generic;
using RimWorld;
using RimWorld.Planet;
using Verse;
using UnityEngine;
using VFECore.Abilities;

namespace AbilitiesExtended
{
	public class CompAbilityEffect_CuttingDash : CompAbilityEffect_WithDest
	{
		//public CompProperties_CuttingDash CompProp => (CompProperties_CuttingDash)props;

		public override void Apply(LocalTargetInfo target, LocalTargetInfo dest)
		{
			Map map = Caster.Map;
			DashingPawn jumpingPawn = (PawnFlyer_KyokatanaDash)PawnFlyer.MakeFlyer(DefDatabase<ThingDef>.GetNamed("WP_CuttingDashFlyer"), base.CasterPawn, target.Cell);
			jumpingPawn.ability = this.parent;

			if (target.Thing == null)
				jumpingPawn.target = target.CenterVector3;
			else jumpingPawn.target = target.Thing.InteractionCell.ToVector3();

			GenSpawn.Spawn(jumpingPawn, base.Caster.Position, map);
			base.Apply(target, dest);
		}
	}
	
	public class PawnFlyer_KyokatanaDash : DashingPawn
    {
		public List<string> HitPawns = new List<string>();
		//need to erase the list on landing/starting flight
        public override Vector3 DrawPos => position;

        public override void SpawnSetup(Map map, bool respawningAfterLoad)
        {
            base.SpawnSetup(map, respawningAfterLoad);
            if (!respawningAfterLoad)
            {
                var a = 0.5f;
                ticksFlightTime = a.SecondsToTicks();
                ticksFlying = 0;
            }
        }

        public override void DrawAt(Vector3 drawLoc, bool flip = false)
        {
            FlyingPawn.Drawer.renderer.RenderPawnAt(position, direction);
        }

        public override void Tick()
        {
			SearchForTargets(position.ToIntVec3(), 2.9f, Map, FlyingPawn);
            base.Tick();
        }
		
		public void SearchForTargets(IntVec3 center, float radius, Map map, Pawn pawn)
        {       
            IEnumerable<IntVec3> targets = GenRadial.RadialCellsAround(center, radius, true);
            foreach (IntVec3 curCell in targets)
            {
                FleckMaker.ThrowDustPuff(curCell, map, .2f);
                if (curCell.InBounds(map) && curCell.IsValid)
                {
                    Pawn victim = curCell.GetFirstPawn(map);
                    if (victim != null && victim.Faction != pawn.Faction)
                    {
						if (!(HitPawns.Contains(victim.GetUniqueLoadID())))
                        {
							damageEntities(victim, null, 15, WP_DefOf.WP_WhiteHotBurn);           
							HitPawns.Add(victim.GetUniqueLoadID());	//we store the ID of everything hit so that we won't hit twice (or, more accurately, once each tick)
						}
                    }
                }
            }
        }
		
		public void damageEntities(Pawn victim, BodyPartRecord hitPart, int amt, DamageDef type)
        {
            DamageInfo dinfo;
            dinfo = new DamageInfo(type, amt, 2, (float)-1, this.FlyingPawn, hitPart, null, DamageInfo.SourceCategory.ThingOrUnknown);
            dinfo.SetAllowDamagePropagation(false);
            victim.TakeDamage(dinfo);
        }
    }
}